import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import japanize_matplotlib

flnm = input('Data File Name (*.xlsx) = ')
xlsx = pd.ExcelFile(flnm)               
data = pd.read_excel(xlsx)      

fout = open('Results.txt', 'w')
fout.write(f'Input data file = {flnm}\n')

print(data)
fout.write(f'\nData =\n{data}\n')

N = data.values
row_names = N[:,0]
col_names = data.keys()[1:]
N = N[:,1:]
sum_row = np.sum(N, axis=1)
sum_col = np.sum(N, axis=0)
n = np.sum(N)
P = N/n
I, J = np.shape(P)
print('\nP =')
for i in range(I):
    for j in range(J):
        print(f'{P[i][j]:>10.4f}', end = '')
    print()
print()
    
r = P @ np.ones((J, 1))
c = P.T @ np.ones((I, 1))
print('r =')
for v in r:
    print(f'{v[0]:>10.4f}')
    
print('\nc =')
for v in c:
    print(f'{v[0]:>10.4f}')
    
Dr = np.diag(r[:,0])
Dc = np.diag(c[:,0])

Dr_n5 = np.diag(r[:,0]**-0.5)
Dc_n5 = np.diag(c[:,0]**-0.5)
S = Dr_n5 @ (P - r @ c.T) @ Dc_n5
S = np.array(S, dtype='float')

U, Da, Vh = np.linalg.svd(S, full_matrices = False)
ck = [1 if v > 1.0e-6 else 0 for v in Da]
K = sum(ck)
print('\nK =', K)
U = U[:,:K]
Da = np.diag(Da[:K])
V = Vh[:K].T

Phi = Dr_n5 @ U
Gmm = Dc_n5 @ V

print('\nPhi =')
for i, v in zip(row_names, Phi):
    print(f'{i:>20s}', end = '')
    for v1 in v:
        print(f'{v1:>10.4f}', end = '')
    print()

print('\nGmm =\n')
for i, v in zip(col_names, Gmm):
    print(f'{i:>20s}', end = '')
    for v1 in v:
        print(f'{v1:>10.4f}', end = '')
    print()
        
F = Phi @ Da
G = Gmm @ Da
print('\nF =')
for i, v in zip(row_names, F):
    print(f'{i:>20s}', end = '')
    for v1 in v:
        print(f'{v1:>10.4f}', end = '')
    print()
    
print('\nG =')
for i, v in zip(col_names, G):
    print(f'{i:>20s}', end = '')
    for v1 in v:
        print(f'{v1:>10.4f}', end = '')
    print()
    
Dlmbd = Da @ Da

alphas = np.diag(Da)
lambdas = np.diag(Dlmbd)
cum_lambdas = lambdas.cumsum()
pcum_lambdas = cum_lambdas / cum_lambdas[-1]
p_lambdas = lambdas / cum_lambdas[-1]

print('\nPrincipal inertias')
print(f'{"Axis ":>10s}{"Principal inertia":>20s}{"% inertia":>12s}{"Cumulative %":>15s}')
for i in range(K):
    print(f'{i+1:>10d}', end = '')
    print(f'{lambdas[i]:>20.5f}', end = '')
    print(f'{p_lambdas[i]*100:>12.1f}', end = '')
    print(f'{pcum_lambdas[i]*100:>15.1f}')
print(f'{"Total":>10s}{cum_lambdas[-1]:>20.5f}' )
print()

fout.write('\nPrincipal inertias\n')
fout.write(f'{"Axis ":>10s}{"Principal inertia":>20s}' +\
           f'{"% inertia":>12s}{"Cumulative %":>15s}\n')
for i in range(K):
    fout.write(f'{i+1:>10d}')
    fout.write(f'{lambdas[i]:>20.5f}')
    fout.write(f'{p_lambdas[i]*100:>12.1f}')
    fout.write(f'{pcum_lambdas[i]*100:>15.1f}\n')
fout.write(f'{"Total":>10s}{cum_lambdas[-1]:>20.5f}\n' )

plt.bar(np.arange(K),lambdas)
plt.xticks(np.arange(K), np.arange(K)+1)
plt.xlabel('Axis', fontsize = 18)
plt.ylabel('Principal inertia', fontsize = 18)
plt.tight_layout()
plt.show()

#
#            Row items/principal coordinates
#            Column items/standard coordinates
#
dim1 = 1
dim2 = 2 if K > 1 else 1
while True:
    plt.figure(figsize = (7,7))
    plt.title('Row items: Principal coordinates\n' +
             'Column items: Standard coordinates')
    for i in range(I):
        plt.plot(F[i][dim1-1], F[i][dim2-1], 'o', c = 'g')
        plt.text(F[i][dim1-1], F[i][dim2-1], row_names[i])
    for j in range(J):
        plt.plot(Gmm[j][dim1-1], Gmm[j][dim2-1], '^', c = 'orange')
        plt.text(Gmm[j][dim1-1], Gmm[j][dim2-1], col_names[j])
    v1 = np.max([F[:,dim1-1], F[:,dim2-1]])
    v2 = np.max([-F[:,dim1-1], -F[:,dim2-1]])
    v3 = np.max([Gmm[:,dim1-1], Gmm[:,dim2-1]])
    v4 = np.max([-Gmm[:,dim1-1], -Gmm[:,dim2-1]])
    max_v = np.max([v1, v2, v3, v4])
    plt.plot([-max_v, max_v], [0, 0], ls = ':', c = 'y')
    plt.plot([0, 0], [-max_v, max_v], ls = ':', c = 'y')
    plt.xlabel(f'Axis {dim1}: {lambdas[dim1-1]:.4f} ({100*p_lambdas[dim1-1]:.1f}%)')
    plt.ylabel(f'Axis {dim2}: {lambdas[dim2-1]:.4f} ({100*p_lambdas[dim2-1]:.1f}%)')
    plt.show()
    print('Row items/principal coordinates: Column items/standard coordinates')
    dim1 = int(input('x-axis = '))
    if dim1 < 1:
        break
    if dim1 > K:
        dim1 = K
    dim2 = int(input('y-axis = '))
    if dim2 < 1:
        dim2 = 1
    if dim2 > K:
        dim2 = K

#
#        Row items/standard coordinates  
#        Column items/principal coordinates
#
dim1 = 1
dim2 = 2 if K > 1 else 1
while True:
    plt.figure(figsize = (7,7))
    plt.title('Row items: Standard coordinates\n' +
             'Column items: Principal coordinates')
    for i in range(I):
        plt.plot(Phi[i][dim1-1], Phi[i][dim2-1], 'o', c = 'g')
        plt.text(Phi[i][dim1-1], Phi[i][dim2-1], row_names[i])
    for j in range(J):
        plt.plot(G[j][dim1-1], G[j][dim2-1], '^', c = 'orange')
        plt.text(G[j][dim1-1], G[j][dim2-1], col_names[j])
    v1 = np.max([Phi[:,dim1-1], Phi[:,dim2-1]])
    v2 = np.max([-Phi[:,dim1-1], -Phi[:,dim2-1]])
    v3 = np.max([G[:,dim1-1], G[:,dim2-1]])
    v4 = np.max([-G[:,dim1-1], -G[:,dim2-1]])
    #print('v4 =', v4)
    #print(-Gmm[:,0:1])
    max_v = np.max([v1, v2, v3, v4])
    plt.plot([-max_v, max_v], [0, 0], ls = ':', c = 'y')
    plt.plot([0, 0], [-max_v, max_v], ls = ':', c = 'y')
    plt.xlabel(f'Axis {dim1}: {lambdas[dim1-1]:.4f} ({100*p_lambdas[dim1-1]:.1f}%)')
    plt.ylabel(f'Axis {dim2}: {lambdas[dim2-1]:.4f} ({100*p_lambdas[dim2-1]:.1f}%)')
    plt.show()
    print('Row items/standard coordinates: Column items/principal coordinates')
    dim1 = int(input('x-axis = '))
    if dim1 < 1:
        break
    if dim1 > K:
        dim1 = K
    dim2 = int(input('y-axis = '))
    if dim2 < 1:
        dim2 = 1
    if dim2 > K:
        dim2 = K

#
#                 Symmetric Map
#
dim1 = 1
dim2 = 2 if K > 1 else 1
while True:
    plt.figure(figsize = (7,7))
    plt.title('Row items: Principal coordinates\n' +
             'Column items: Principal coordinates')
    for i in range(I):
        plt.plot(F[i][dim1-1], F[i][dim2-1], 'o', c = 'g')
        plt.text(F[i][dim1-1], F[i][dim2-1], row_names[i])
    for j in range(J):
        plt.plot(G[j][dim1-1], G[j][dim2-1], '^', c = 'orange')
        plt.text(G[j][dim1-1], G[j][dim2-1], col_names[j])
    v1 = np.max([F[:,dim1-1], F[:,dim2-1]])
    v2 = np.max([-F[:,dim1-1], -F[:,dim2-1]])
    v3 = np.max([G[:,dim1-1], G[:,dim2-1]])
    v4 = np.max([-G[:,dim1-1], -G[:,dim2-1]])
    max_v = np.max([v1, v2, v3, v4])
    plt.plot([-max_v, max_v], [0, 0], ls = ':', c = 'y')
    plt.plot([0, 0], [-max_v, max_v], ls = ':', c = 'y')
    plt.xlabel(f'Axis {dim1}: {lambdas[dim1-1]:.4f} ({100*p_lambdas[dim1-1]:.1f}%)')
    plt.ylabel(f'Axis {dim2}: {lambdas[dim2-1]:.4f} ({100*p_lambdas[dim2-1]:.1f}%)')
    plt.show()
    print('Row and Column items/principal coordinates')
    dim1 = int(input('x-axis = '))
    if dim1 < 1:
        break
    if dim1 > K:
        dim1 = K
    dim2 = int(input('y-axis = '))
    if dim2 < 1:
        dim2 = 1
    if dim2 > K:
        dim2 = K

fout.close()
print('Results.txt was saved.')
